@@ -24,6 +24,10 @@
 #ifndef L_H
 #define L_H
 
+#ifdef _OPENMP
+#include <omp.h>
+#endif
+
 #include <iomanip>          //for manipulating output such as setprecision
 #include <fstream>          //for file input output
 #include <string.h>         //for string functions such as strcpy
@@ -31,12 +35,15 @@
 #include <iostream>         //for ostrstream
 
 #include <math.h>
+#include<vector>
 
 #include "Lglobals.h"           //for global variables
 #include "Lmisc.h"              //things like sn or LOG
 #include "Lgamma.h"             //incomplete gamma function code
 //#include "Lprecomp.h"
 #include "Lriemannsiegel.h"     //Riemann Siegel formula
+#include "Lriemannsiegel_blfi.h" //Hiary's Riemann Siegel formula using band limited interpolation
+
 
 
 //-----THE L Function Class-----------------------------------
@@ -125,7 +132,7 @@ public:
 
 
     //-----Constructor: initialize the L-function from given data------------------
-    L_function (char *NAME, int what_type, int N, ttype *coeff, long long Period,
+    L_function (const char *NAME, int what_type, int N, ttype *coeff, long long Period,
     Double q,  Complex w, int A, Double *g, Complex *l,
     int n_poles, Complex *p, Complex *r)
     {
@@ -187,7 +194,7 @@ public:
     }
 
     //-----Constructor: initialize the L-function from given data------------------
-    L_function (char *NAME, int what_type, int N, ttype *coeff, long long Period,
+    L_function (const char *NAME, int what_type, int N, ttype *coeff, long long Period,
     Double q,  Complex w, int A, Double *g, Complex *l)
     {
         if(my_verbose>1)
@@ -381,8 +388,9 @@ public:
 
 
     //-----addition operator--------------------
-    //returns the L-function whose basic data (funct eqn, dirichlet coefficients) 
+    //returns the 'L-function' whose basic data (funct eqn, dirichlet coefficients) 
     //is that of this added to that of L
+    //not particularly useful
     L_function operator + (const L_function &L)
     {
         L_function L2;
@@ -423,9 +431,10 @@ public:
         return L2;
 
     }
-    //-----addition operator--------------------
+    //-----multiplication operator--------------------
     //returns the L-function whose basic data (funct eqn, dirichlet coefficients) 
-    //is that of this added to that of L
+    //is that of this times the scalar t
+    //not particularly useful
     L_function operator * (double t)
     {
         L_function L2;
@@ -492,18 +501,27 @@ public:
 
     //#include "Lvalue.h"         //value via Riemann sum, via gamma sum, various options for value
     Complex find_delta(Complex s,Double g);
-    Complex value_via_Riemann_sum(Complex s,char *return_type="pure");
-    Complex value_via_gamma_sum(Complex s, char *return_type="pure");
-    Complex value(Complex s, int derivative = 0, char *return_type="pure");
+    Complex value_via_Riemann_sum(Complex s, const char *return_type="pure");
+    Complex value_via_gamma_sum(Complex s, const char *return_type="pure");
+    Complex value(Complex s, int derivative = 0, const char *return_type="pure");
 
     //#include "Lfind_zeros.h"    //finding zeros routine
     Double zeros_zoom_brent(Double L1, Double L2, Double u, Double v);
-    void find_zeros(Double t1, Double t2, Double step_size, char* filename="cout", char* message_stamp="");
-    void find_zeros_via_gram(Double t1, Long count=0,Double max_refine=1025,char* filename="cout", char* message_stamp="");
+    void find_zeros(Double t1, Double t2, Double step_size, const char* filename="cout", const char* message_stamp="");
+    void  find_zeros_v(Double t1, Double t2, Double step_size,  vector<Double> &result);//This is the same as above function
+    void find_zeros_via_gram(Double t1, Long count=0,Double max_refine=1025, const char* filename="cout", const char* message_stamp="");
     int compute_rank(bool print_rank=false);
     void verify_rank(int rank);
-    void find_zeros_via_N(Long count=0,bool do_negative=true,Double max_refine=1025, int rank=-1,char* filename="cout", char* message_stamp="");
-    void find_zeros_elaborate(Double t1, Long count=0,Double max_refine=1025,char* filename="cout", char* message_stamp="");
+    void find_zeros_via_N(Long count=0,bool do_negative=true,Double max_refine=1025, int rank=-1, bool test_explicit_formula=false, const char* filename="cout", const char* message_stamp="");
+    void find_zeros_via_N_v(Long count,bool do_negative,Double max_refine, int rank, bool test_explicit_formula, vector<Double>  &result);
+    void find_zeros_elaborate(Double t1, Long count=0,Double max_refine=1025, const char* filename="cout", const char* message_stamp="");
+
+    //#include "Ldokchitser.h"    //dokchitser algorithm for what he calls phi(t), i.e. inverse
+    void phi_series(int precision);
+
+    //#include "Lexplicit_formula.h"
+    int dirichlet_coeffs_log_diff(int num_coeff, Complex *c);
+    int test_explicit_formula(Double A, Double x_0, Double *zero_table, int number_zeroes, Complex *c, int num_coeffs);
 
 
 };
@@ -518,5 +536,9 @@ public:
 //#include "Ltaylor_series.h" //for computing taylor series for Dirichlet series
 #include "Lvalue.h"         //value via Riemann sum, via gamma sum, various options for value
 #include "Lfind_zeros.h"    //finding zeros routine
+#include "Ldokchitser.h"    //dokchitser algorithm for what he calls phi(t), i.e. inverse
+                            //mellin transform
+
+#include "Lexplicit_formula.h" //for testing zeros with the explicit formula
 
 #endif
