@@ -29,6 +29,7 @@
 #include "Lmisc.h"
 #include <iomanip>          //for manipulating output such as setprecision
 #include <iostream>         //for ostrstream
+#include <cstring>
 //using namespace std;
 
 
@@ -43,7 +44,7 @@ Complex GAMMA (ttype z,ttype2 delta);
 
 //computes G(z,w)
 template <class ttype>
-Complex inc_GAMMA (ttype z,ttype w, char *method="temme", ttype exp_w = 0, bool recycle=false);  //computes G(z,w)
+Complex inc_GAMMA (ttype z,ttype w, const char *method="temme", ttype exp_w = 0, bool recycle=false);  //computes G(z,w)
 
 template <class ttype>
 ttype asympt_GAMMA (ttype z,ttype w, ttype exp_w = 0, bool recycle=false);  //computes G(z,w) via asymptotic series
@@ -67,7 +68,7 @@ Complex erfc(Complex z);
 Complex erfc2(Complex z);
 
 template <class ttype>
-Complex gamma_sum(Complex s, int what_type,ttype *coeff, int N, Double g, Complex l, Double Q, Long Period, Complex delta=1, char *method="temme");
+Complex gamma_sum(Complex s, int what_type,ttype *coeff, int N, Double g, Complex l, Double Q, Long Period, Complex delta=1, const char *method="temme");
 //computes a sum of G(z,w)'s (as in (3.3.20)).
 
 Complex exp_recycle();
@@ -229,7 +230,7 @@ Complex GAMMA (ttype z1, ttype2 delta)
 //value exp_w which holds exp(-w)
 //computes G(z,w), so there's an extra w^(-z) factor.
 template <class ttype>
-Complex inc_GAMMA (ttype z,ttype w, char *method="temme", ttype exp_w = 0, bool recycle=false)
+Complex inc_GAMMA (ttype z,ttype w, const char *method="temme", ttype exp_w = 0, bool recycle=false)
 {
 
     Complex G;
@@ -240,7 +241,7 @@ Complex inc_GAMMA (ttype z,ttype w, char *method="temme", ttype exp_w = 0, bool
     if(my_verbose>2) cout << "inc_GAMMA called. G(" << z<< " , " << w << ")" << endl;
 
     //if(my_norm(z)<tolerance_sqrd){
-    if(my_norm(z)<.1){
+    if(my_norm(z)<.01){
         return cfrac_GAMMA(z,w,exp_w,recycle);
     }
 
@@ -603,7 +604,7 @@ ttype comp_inc_GAMMA (ttype z,ttype w,ttype exp_w = 0, bool recycle=false)  //co
 }
 
 template <class ttype>
-Complex gamma_sum(Complex s, int what_type, ttype *coeff, int N, Double g, Complex l, Double Q, Long Period, Complex delta=1, char *method="temme")
+Complex gamma_sum(Complex s, int what_type, ttype *coeff, int N, Double g, Complex l, Double Q, Long Period, Complex delta=1, const char *method="temme")
 {
     Complex SUM=0;
 
@@ -703,7 +704,7 @@ Complex gamma_sum(Complex s, int what_type, ttype *coeff, int N, Double g, Compl
             //which is more time consuming
             if(is_z_real&&is_w_real){
                 G=inc_GAMMA(Double(real(z)),Double(real(w)),method,Double(real(exp_w)),true);
-                if (my_verbose>2) cout << "GAMMA SUM = " << G << endl;
+                if (my_verbose>2) cout << "GAMMA SUM with doubles = " << G << endl;
                 //cout<<"both z and w are real\n";
             }
             else{
