@@ -26,12 +26,12 @@
 #ifndef Lriemannsiegel_H
 #define Lriemannsiegel_H
 
-#include "Lglobals.h"           //for global variables
-#include "Lgamma.h"           //for global variables
-#include "Lmisc.h"           //for global variables
+//#include "Lglobals.h"           //for global variables
+//#include "Lgamma.h"           //for global variables
+//#include "Lmisc.h"           //for global variables
 
-Complex Zeta(Complex s,char *return_type="pure");
-Double C(int i, Double p);
+Complex Zeta(Complex s, const char *return_type="pure");
+Double C(int n, Double *powers_z);
 Complex siegel(Complex s, int N=10);
 
 #endif
