@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2001,2002,2003,2004 Michael Rubinstein
+Copyright (C) Michael Rubinstein
 
 This file is part of the L-function package L.
 
@@ -42,6 +42,8 @@ int main (int argc, char *argv[])
     char data_filename[1000]; //filename of file containing data for L-function.
     char data_filename2[1000]; //filename of file containing data for L-function.
     int number_coeff_print=30;
+    bool do_hardy=false;
+    bool do_explicit=false; //whether to test the explicit formula when looking for zeros
 
     //used for elliptic curves
     // y^2 + a1 xy + a3 y = x^3 + a2 x^2 + a4 x + a6
@@ -118,6 +120,7 @@ int main (int argc, char *argv[])
         #endif
         //step_size=atof(args_info.stepsize_arg);
     }
+    if(args_info.explicit_given) do_explicit=true; /* Whether to check the explicit formula  */
     if(args_info.value_given){ /* Whether value was given.  */
         do_values=true;
         if(!args_info.x_given||!args_info.y_given){
@@ -172,6 +175,9 @@ int main (int argc, char *argv[])
     if(args_info.number_samples_given){ /* Whether derivative was given.  */
         count=args_info.number_samples_arg;
     }
+    if(args_info.hardy_given){ /* Whether Hardy Z option was given.  */
+        do_hardy=true;
+    }
 
     if(args_info.elliptic_curve_given){ /* Whether elliptic-curve was given.  */
         do_elliptic_curve=true;
@@ -284,7 +290,35 @@ int main (int argc, char *argv[])
         D1=strtoll(args_info.start_arg,0,10); D2=strtoll(args_info.finish_arg,0,10);
     }
 
-
+    #ifdef _OPENMP
+        if(args_info.openmp_given) /* whether a number of threads was specified */
+        {
+            /* get the total number of CPUs/cores available for OpenMP */
+            int    NCPU = omp_get_num_procs();
+            if (args_info.openmp_arg>NCPU){
+                cout << "ERROR: You only have " << NCPU << " processors available." << endl;
+                cout << "Specify fewer processors" << endl;
+                exit(1);
+            }
+            omp_set_num_threads(args_info.openmp_arg);
+        }
+        else //default is 1
+            omp_set_num_threads(1);
+    #endif
+    #ifndef _OPENMP
+        if(args_info.openmp_given){ /* whether a number of threads was specified */
+            cout << endl;
+            cout << "ERROR: lcalc has not been compiled with openmp enabled." << endl;
+            cout << "You need to uncomment the line: " << endl;
+            cout << "#OPENMP_FLAG = -fopenmp" << endl;
+            cout << "in the Makefile, and recompile by typing:"<< endl;
+            cout << endl;
+            cout << "make clean" << endl;
+            cout << "make" << endl;
+            cout << endl;
+            exit(1);
+        }
+    #endif
 
     //placed here rather than at top since
     //precision depends on user input.
@@ -301,6 +335,21 @@ int main (int argc, char *argv[])
         //DIGITS3=(int)(DIGITS3/pow(2.,global_derivative)); // now taken care of in Lvalue.h
     }
 
+    if(do_zeros){
+        input_mean_spacing_given = (6.28/log(count*1.+100))*2./DIGITS; //only used
+        //for the zeta funciton, and then in the band limited interpolation scheme. Is a
+        //rough estimate for the average distance between sample points.
+        //Dividing by DIGITS/2 more than accounts for searching for and then zooming in
+        //on zeros.
+        //cout << " input_mean_spacing_given " << input_mean_spacing_given << endl;
+
+    }
+
+    if(args_info.use_dirichlet_series_given){ /* Whether to compute just using the Dirichlet series */
+        only_use_dirichlet_series=true;
+        N_use_dirichlet_series=args_info.use_dirichlet_series_arg; //how many terms to use
+    }
+
 
     if(args_info.verbose_given){ /* Whether vebosity level was specified.  */
         my_verbose=args_info.verbose_arg;
@@ -363,7 +412,7 @@ int main (int argc, char *argv[])
 
             t2=.5; //t2=.5 because of the GAMMA(s+1/2)
 
-            pari_init(1000000,2); 
+            pari_init(1000000000,2);
             //pari_init_opts(400000000,2,INIT_DFTm); // the last option is to prevent
             //pari from giving its interrupt signal when its elliptic curve a_p
             //algorithm is called and interrupted with ctrl-c. Requires a more current
@@ -404,7 +453,7 @@ int main (int argc, char *argv[])
 
         if(my_verbose>1) cout << "T = " << T << endl;
 
-        if(count>0)
+        if(count>0&&!args_info.number_samples_given)
         T=T+(count+100)*Pi/log(T+3);
 
         if(my_verbose>1) cout << "T = " << T << endl;
@@ -415,8 +464,10 @@ int main (int argc, char *argv[])
         N_terms = Int(2.3 * DIGITS*tmp_Q/real(delta));
         do{
             N_terms=(int)(N_terms*1.3);
+            if(my_verbose>4) cout << "N_terms to precompute = " << N_terms << endl;
         }while(N_terms*real(delta)/tmp_Q-log((1.+t2)*N_terms)<2.3*DIGITS);
-        N_terms=(int)(N_terms*1.3);
+        N_terms=(int)(N_terms*1.3+40);
+        if(my_verbose>4) cout << "N_terms to precompute = " << N_terms << endl;
 
 
 
@@ -478,46 +529,55 @@ int main (int argc, char *argv[])
             switch(twist_type){
                 case 'q':
                     if(do_compute_rank)
-                        quadratic_twists(D1,D2,x,y,0,0,"values and ranks",do_only_even_twists);
+                        quadratic_twists(D1,D2,x,y,0,0,"values and ranks",do_only_even_twists,do_explicit);
                     else if(limit_the_rank)
-                        quadratic_twists(D1,D2,x,y,0,0,"values and ranks",do_only_even_twists,rank);
+                        quadratic_twists(D1,D2,x,y,0,0,"values and ranks",do_only_even_twists,rank,do_explicit);
                     else
-                        quadratic_twists(D1,D2,x,y,0,0,"values",do_only_even_twists);
+                        quadratic_twists(D1,D2,x,y,0,0,"values",do_only_even_twists,do_explicit);
                     break;
                 case 'p':
                     if(do_compute_rank)
-                        all_twists(D1,D2,x,y,0,0,"values and ranks",0,print_character);
+                        all_twists(D1,D2,x,y,0,0,"values and ranks",0,print_character,do_explicit);
                     else
-                        all_twists(D1,D2,x,y,0,0,"values",0,print_character);
+                        all_twists(D1,D2,x,y,0,0,"values",0,print_character,do_explicit);
                     break;
                 case 'a':
                     if(do_compute_rank)
-                        all_twists(D1,D2,x,y,0,0,"values and ranks",1,print_character);
+                        all_twists(D1,D2,x,y,0,0,"values and ranks",1,print_character,do_explicit);
                     else
-                        all_twists(D1,D2,x,y,0,0,"values",1,print_character);
+                        all_twists(D1,D2,x,y,0,0,"values",1,print_character,do_explicit);
                     break;
                 case 'A':
                     if(do_compute_rank)
-                        all_twists(D1,D2,x,y,0,0,"values and ranks",2,print_character);
+                        all_twists(D1,D2,x,y,0,0,"values and ranks",2,print_character,do_explicit);
                     else
-                        all_twists(D1,D2,x,y,0,0,"values",2,print_character);
+                        all_twists(D1,D2,x,y,0,0,"values",2,print_character,do_explicit);
                     break;
                 case 'g':
                     if(do_compute_rank)
-                        all_twists(D1,D2,x,y,0,0,"values and ranks",-1,print_character);
+                        all_twists(D1,D2,x,y,0,0,"values and ranks",-1,print_character,do_explicit);
                     else
-                        all_twists(D1,D2,x,y,0,0,"values",-1,print_character);
+                        all_twists(D1,D2,x,y,0,0,"values",-1,print_character,do_explicit);
                     break;
                 case 'c':
                     if(do_compute_rank)
-                        all_twists(D1,D2,x,y,0,0,"values and ranks",-2,print_character);
+                        all_twists(D1,D2,x,y,0,0,"values and ranks",-2,print_character,do_explicit);
                     else
-                        all_twists(D1,D2,x,y,0,0,"values",-2,print_character);
+                        all_twists(D1,D2,x,y,0,0,"values",-2,print_character,do_explicit);
                     break;
             }
         }
-        else if(file_of_s_values) compute_values(x,y,s_file_name);
-        else compute_values(x,y,"",x2,y2,count);
+        else if(file_of_s_values){
+             if(!do_hardy) compute_values(x,y,"pure",s_file_name);
+             else compute_values(x,y,"rotated pure",s_file_name);
+        }
+        else{
+            input_mean_spacing_given=(y2-y)/count; //used in Riemann Siegel band limited routine
+            //cout << " input_mean_spacing_given " << input_mean_spacing_given << endl;
+            //for zeta, if applicable.
+            if(!do_hardy) compute_values(x,y,"pure","",x2,y2,count);
+            else compute_values(x,y,"rotated pure","",x2,y2,count);
+        }
 
     }
     else if(do_zeros){
@@ -525,30 +585,30 @@ int main (int argc, char *argv[])
             switch(twist_type){
                 case 'q':
                     if(limit_the_rank)
-                    quadratic_twists(D1,D2,x,y,count,step_size,"zeros and ranks",do_only_even_twists,rank);
+                    quadratic_twists(D1,D2,x,y,count,step_size,"zeros and ranks",do_only_even_twists,do_explicit,rank);
                     else
-                    quadratic_twists(D1,D2,x,y,count,step_size,"zeros",do_only_even_twists);
+                    quadratic_twists(D1,D2,x,y,count,step_size,"zeros",do_only_even_twists,do_explicit);
                     break;
                 case 'p':
-                    all_twists(D1,D2,x,y,count,step_size,"zeros",0,print_character);
+                    all_twists(D1,D2,x,y,count,step_size,"zeros",0,print_character,do_explicit);
                     break;
                 case 'a':
-                    all_twists(D1,D2,x,y,count,step_size,"zeros",1,print_character);
+                    all_twists(D1,D2,x,y,count,step_size,"zeros",1,print_character,do_explicit);
                     break;
                 case 'A':
-                    all_twists(D1,D2,x,y,count,step_size,"zeros",2,print_character);
+                    all_twists(D1,D2,x,y,count,step_size,"zeros",2,print_character,do_explicit);
                     break;
                 case 'g':
-                    all_twists(D1,D2,x,y,count,step_size,"zeros",-1,print_character);
+                    all_twists(D1,D2,x,y,count,step_size,"zeros",-1,print_character,do_explicit);
                     break;
                 case 'c':
-                    all_twists(D1,D2,x,y,count,step_size,"zeros",-2,print_character);
+                    all_twists(D1,D2,x,y,count,step_size,"zeros",-2,print_character,do_explicit);
                     break;
 
             }
         }
         else{
-            compute_zeros(x,y,step_size,count,rank);
+            compute_zeros(x,y,step_size,count,rank,do_explicit);
         }
     }
     else{ //else allow for printing of characters. "print character" is a dummy char str,
@@ -556,19 +616,19 @@ int main (int argc, char *argv[])
         if(do_twists){
             switch(twist_type){
                 case 'p':
-                    all_twists(D1,D2,x,y,count,step_size,"print character",0,print_character);
+                    all_twists(D1,D2,x,y,count,step_size,"print character",0,print_character,do_explicit);
                     break;
                 case 'a':
-                    all_twists(D1,D2,x,y,count,step_size,"print character",1,print_character);
+                    all_twists(D1,D2,x,y,count,step_size,"print character",1,print_character,do_explicit);
                     break;
                 case 'A':
-                    all_twists(D1,D2,x,y,count,step_size,"print character",2,print_character);
+                    all_twists(D1,D2,x,y,count,step_size,"print character",2,print_character,do_explicit);
                     break;
                 case 'g':
-                    all_twists(D1,D2,x,y,count,step_size,"print character",-1,print_character);
+                    all_twists(D1,D2,x,y,count,step_size,"print character",-1,print_character,do_explicit);
                     break;
                 case 'c':
-                    all_twists(D1,D2,x,y,count,step_size,"print character",-2,print_character);
+                    all_twists(D1,D2,x,y,count,step_size,"print character",-2,print_character,do_explicit);
                     break;
 
             }
