@@ -475,13 +475,13 @@ Double L_1_chi(long long d)
     {
         if(d<0){
             for(j=1;j<=-d;j++) L=L+j*my_kronecker(d,j);
-            L=-L*Pi*exp(-1.5*log(Double(-d)));
+            L=-L*Pi*exp(-((Double)3)/2*log(Double(-d)));
         }
         else{
             for(j=1;j<=d-1;j++){
                 L=L+my_kronecker(d,j)*log(sin(j*Pi/d));
             }
-            L=-L*exp(-.5*log(Double(d)));
+            L=-L*exp(-log(Double(d))/2);
         }
 
         return L;
@@ -507,7 +507,7 @@ int class_number(long long d)
         if(d<0){
             if(d==-4)w=4;
             if(d==-3)w=6;
-            h=Int(w*exp(.5*log(Double(-d)))*L_1_chi(d)/(2*Pi)+.000001);
+            h=Int(w*exp(log(Double(-d))/2)*L_1_chi(d)/(2*Pi)+.000001);
         }
         else{
             u=0;
@@ -519,8 +519,8 @@ int class_number(long long d)
                 t=(long long)x;
             }while(x-t>.00001);
             cout << "t = " << t << "  u = " << u << endl;
-            epsilon=.5*(t+u*sqrt(1.*d));
-            h=Int(exp(.5*log(Double(d)))*L_1_chi(d)/log(epsilon)+.000001);
+            epsilon=(t+u*sqrt(1.*d))/2;
+            h=Int(exp(log(Double(d))/2)*L_1_chi(d)/log(epsilon)+.000001);
         }
         return h;
     }
@@ -543,8 +543,8 @@ void ramanujan_tau(Double *c0, int N_terms)
     c= new Double[i+2];
     c1= new Double[N_terms+2];
 
-    for(i=0;i<=N_terms;i++)c0[i]=0.;
-    c0[0]=1.;
+    for(i=0;i<=N_terms;i++)c0[i]=0;
+    c0[0]=1;
 
     for(n=0;n<=x;n++){c[n]=(2*n+1);if(n%2==1)c[n]=-c[n];}
 
@@ -649,8 +649,8 @@ int prim_root(long long p, int alpha)
 
     }while(!is_primitive&&a<p-1);
 
-    for(k=0;k<=29;k++) delete(factors[k]);
-    delete (factors);
+    for(k=0;k<=29;k++) delete[](factors[k]);
+    delete[] (factors);
 
     if(alpha==1) return a;
 
@@ -801,13 +801,13 @@ int characters()
 
 
         for(n=1;n<=q-1;n++){
-            chi[n]=1.;
-            chi_1[n]=1.;
+            chi[n]=1;
+            chi_1[n]=1;
         }
-        chi[0]=0.;
-        chi_1[0]=0.;
-        chi[q]=0.;
-        chi_1[q]=0.;
+        chi[0]=0;
+        chi_1[0]=0;
+        chi[q]=0;
+        chi_1[q]=0;
 
         q_1=q;
 
@@ -884,8 +884,8 @@ int characters()
         }
 
         for(n=0;n<=q;n++){
-            if(gcd(n,q)>1)chi[n]=0.;
-            if(gcd(n,q_1)>1)chi_1[n]=0.;
+            if(gcd(n,q)>1)chi[n]=0;
+            if(gcd(n,q_1)>1)chi_1[n]=0;
         }
 
         cout << "inducing q_1 = " << q_1 << endl;
