@@ -116,7 +116,7 @@ higher degree:
 //the first count zeros above x.
 
 
-int quadratic_twists(Long D1, Long D2,Double x,Double y,int count,Double step_size,char *what_to_do,bool do_only_even_twists,int desired_rank)
+int quadratic_twists(Long D1, Long D2,Double x,Double y,int count,Double step_size,const char *what_to_do,bool do_only_even_twists,bool test_explicit_formula,int desired_rank)
 {
 
     int k,n;
@@ -259,8 +259,8 @@ int quadratic_twists(Long D1, Long D2,Double x,Double y,int count,Double step_si
         {
             //if(my_kronecker(d,d-1)==1) tmp_l[1]=0.;
             //this is -1 for d<0 and 1 for d>0, so ... no need for the call to kronecker
-            if(d>0) tmp_l[1]=0.;
-            else tmp_l[1]=.5;
+            if(d>0) tmp_l[1]=0;
+            else tmp_l[1]=(Double)1/2;
             tmp_Q=sqrt(r/Pi);
 
             //C = 2*DIGITS2*log(10.);
@@ -446,7 +446,7 @@ int quadratic_twists(Long D1, Long D2,Double x,Double y,int count,Double step_si
                             int_L2.find_zeros(x,y,step_size,"cout",message_stamp);
                             else
                             //int_L2.find_zeros_via_gram(x,count,step_size,"cout",message_stamp);
-                            int_L2.find_zeros_via_N(count,false,step_size,-1,"cout",message_stamp);
+                            int_L2.find_zeros_via_N(count,false,step_size,-1,test_explicit_formula,"cout",message_stamp);
                             //int_L2.find_zeros_elaborate(x,count,step_size,"cout",message_stamp);
                             break;
                         case 2:
@@ -454,7 +454,7 @@ int quadratic_twists(Long D1, Long D2,Double x,Double y,int count,Double step_si
                             Double_L2.find_zeros(x,y,step_size,"cout",message_stamp);
                             else
                             //Double_L2.find_zeros_via_gram(x,count,step_size,"cout",message_stamp);
-                            Double_L2.find_zeros_via_N(count,false,step_size,-1,"cout",message_stamp);
+                            Double_L2.find_zeros_via_N(count,false,step_size,-1,test_explicit_formula,"cout",message_stamp);
                             //Double_L2.find_zeros_elaborate(x,count,step_size,"cout",message_stamp);
                             break;
                         case 3:
@@ -462,7 +462,7 @@ int quadratic_twists(Long D1, Long D2,Double x,Double y,int count,Double step_si
                             Complex_L2.find_zeros(x,y,step_size,"cout",message_stamp);
                             else
                             //Complex_L2.find_zeros_via_gram(x,count,step_size,"cout",message_stamp);
-                            Complex_L2.find_zeros_via_N(count,true,step_size,-1,"cout",message_stamp);
+                            Complex_L2.find_zeros_via_N(count,true,step_size,-1,test_explicit_formula,"cout",message_stamp);
                             break;
                     }
 
@@ -551,7 +551,7 @@ int quadratic_twists(Long D1, Long D2,Double x,Double y,int count,Double step_si
 
 //twist_type: -1 just one complex primitive twist, -2 all non-real primitive twists, 0 all primitive twists, 1 all twists with
 //conjugate pairs only appearing once, 2 all twists with conjugate pairs appearing twice
-int all_twists(Long D1, Long D2,Double x,Double y,int count,Double step_size,char *what_to_do,int twist_type,int print_character)
+int all_twists(Long D1, Long D2,Double x,Double y,int count,Double step_size,const char *what_to_do,int twist_type,int print_character,bool test_explicit_formula)
 {
 
     //variables for twisting
@@ -603,7 +603,7 @@ int all_twists(Long D1, Long D2,Double x,Double y,int count,Double step_size,cha
     Complex s;
 
     Long d;
-    char *tmp_name="L_chi";
+    const char *tmp_name="L_chi";
 
     i=0;
 
@@ -759,7 +759,7 @@ int all_twists(Long D1, Long D2,Double x,Double y,int count,Double step_size,cha
 
 
         r=d; // a bit redundant since we are using q,r,d for the 
-             // same value. But this allows me to mouse quadratic_twists
+             // same value. But this allows me to mouse Quadratic_twists
 
 
 // xxxxxxxxxxxx trivial character do not compute...
@@ -824,10 +824,10 @@ if(is_fresh&&(is_primitive || !only_do_primitive))
             chi[n]=1.;
             chi_1[n]=1.;
         }
-        chi[0]=0.;
-        chi_1[0]=0.;
-        chi[q]=0.;
-        chi_1[q]=0.;
+        chi[0]=0;
+        chi_1[0]=0;
+        chi[q]=0;
+        chi_1[q]=0;
 
         q_1=q;
 
@@ -905,8 +905,8 @@ if(is_fresh&&(is_primitive || !only_do_primitive))
         }
 
         for(n=0;n<=q;n++){
-            if(gcd(n,q)>1)chi[n]=0.;
-            if(gcd(n,q_1)>1)chi_1[n]=0.;
+            if(gcd(n,q)>1)chi[n]=0;
+            if(gcd(n,q_1)>1)chi_1[n]=0;
         }
 
         //cout << "inducing q_1 = " << q_1 << endl;
@@ -962,12 +962,12 @@ if(is_fresh&&(is_primitive || !only_do_primitive))
         if(what_type==-1) // -1 is for zeta
         {
             if(abs(chi_1[q_1-1]-1)<.1){
-                tmp_l[1]=0.;
-                tmp_OMEGA=tau_chi/sqrt(1.*q_1);
+                tmp_l[1]=0;
+                tmp_OMEGA=tau_chi/sqrt((Double)q_1);
             }
             else{
-                tmp_l[1]=.5;
-                tmp_OMEGA=tau_chi/(I*sqrt(1.*q_1));
+                tmp_l[1]=(Double)1/2;
+                tmp_OMEGA=tau_chi/(I*sqrt((Double)q_1));
 
             }
             tmp_Q=sqrt(q_1/Pi);
@@ -1109,10 +1109,10 @@ if(is_fresh&&(is_primitive || !only_do_primitive))
                 Complex_L2.find_zeros(x,y,step_size,"cout",message_stamp);
 
                 if(current_L_type==3||max_imag_chi>.1)
-                Complex_L2.find_zeros_via_N(count,true,step_size,-1,"cout",message_stamp);
+                Complex_L2.find_zeros_via_N(count,true,step_size,-1,test_explicit_formula,"cout",message_stamp);
                 //comeplexL zero finder looks for zeros from the origin upwards
                 //else Complex_L2.find_zeros_via_gram(x,count,step_size,"cout",message_stamp);
-                else Complex_L2.find_zeros_via_N(count,false,step_size,-1,"cout",message_stamp);
+                else Complex_L2.find_zeros_via_N(count,false,step_size,-1,test_explicit_formula,"cout",message_stamp);
                 //gram zero finder can start anywhere
                 if(twist_type==-1){M=PHI;m[0]=loop2;m_prime=loop1;}
 
@@ -1174,8 +1174,8 @@ if(is_fresh&&(is_primitive || !only_do_primitive))
 
 
     } // for d
-    for(k=0;k<=29;k++) delete(factors[k]);
-    delete (factors);
+    for(k=0;k<=29;k++) delete[](factors[k]);
+    delete [] (factors);
     delete [] tmp_g;
     delete [] tmp_l;
     delete [] tmp_cmplx;
